/* graphics filter for HP Deskjet printer                     *
*  75,100,150,300 dots per inch	
*  */
#define ALWAYSHEAVY     /* take this out for fine lines in high res. mode */

#include <stdio.h>
#include <sgtty.h>
#include <signal.h>
#define ESC 27          /* ASCII escape   */
#define CR 13		/* Carriage return */
#define LF 10		/* Line feed */

double xmin,xmax,ymin,ymax,xscale,yscale,xdel,ydel;
/* boundaries of plot region, multipliers for converting to pixels, and
   distances between columns and rows */
int density=150;	/* density in dots per inch */
int cols=1200,rows=1200; /* number of columns and rows, default density  */
int rowbytes=150;	/* number of bytes in a row = (cols+7)/8*/
int erase=0;		/* formfeed if erase */
char *image;		/* raster image */
static char mask[8] = {0200,0100,0040,0020,0010,0004,0002,0001 };
/* masks for picking single bits */
double getcoord();
char *malloc();
void cleanup();	/* interrupt handler */

main(argc,argv) char **argv; {

    getopt(argc,argv);	/* set options */

    setup();            /* initialize image, status variables */
    makeplot();         /* read input and make image            */
    putimage();         /* put result to printer                 */
}

getopt(argc,argv) char **argv; {
    if(argc > 1){
        if     (strcmp("-x600" ,argv[1])==0){
	    density=75; cols=rows=600; rowbytes=75;
	}
        else if(strcmp("-x800" ,argv[1])==0){
	    density=100; cols=rows=800; rowbytes=100;
	}
        else if (strcmp("-x1200",argv[1])==0){
	    density=150; cols=rows=1200; rowbytes=150;
	}
        else if(strcmp("-x2400",argv[1])==0){
	    density=300; cols=rows=2400; rowbytes=300;
	}
        else {
	    fprintf(stderr,"Unknown xwidth %s\n",argv[1]);
	    exit(2);
        }
    }
}

setup(){
    xmin = 0.0;         ymin = 0.0;     /* defaults */
    xmax = 1000.0;      ymax = 1000.0;
    xdel = 1.05*(xmax-xmin)/cols;   ydel = 1.05*(ymax-ymin)/rows;
    /* the 1.05 is to allow overlap on right */
    xscale = 1.0/xdel;          yscale = 1.0/ydel;
    image = malloc(rowbytes*rows);
    if( image==NULL ){fprintf(stderr,"Malloc failed\n"); exit(2); }
}

makeplot(){
int code,c,i,j,k,complaints;
double x,y,xcurrent,ycurrent,xnext,ynext;
    complaints=0;
    while( (code = getchar()) != EOF)switch(code){
        case 'e': erase = 1;    /* erase screen */
                break;
        case 's': xmin = getcoord();    /* set up plotting window */
                ymin = getcoord();
                xmax = getcoord();
                ymax = getcoord();
                xdel = 1.05*(xmax-xmin)/cols;	ydel = 1.05*(ymax-ymin)/rows;
                xscale = 1.0/xdel;              yscale = 1.0/ydel;
                break;
        case 'm':                       /* read in current point */
                xcurrent = getcoord();
                ycurrent = getcoord();
                break;
        case 'p':                       /* plot a point */
                xcurrent = getcoord();
                ycurrent = getcoord();
                point(xcurrent,ycurrent);
                break;
        case 'l':                       /* plot a line  */
                xcurrent = getcoord();
                ycurrent = getcoord();
                        /* continue on into case 'n' */
        case 'n':                       /* line from current to next*/
                xnext = getcoord();
                ynext = getcoord();
		line(xcurrent,ycurrent,xnext,ynext);
                xcurrent = xnext;       ycurrent = ynext;
                break;
        case 't':       /* put a label in the graph */
                fprintf(stderr,"don't know how to make labels\n");
                while( (c=getchar()) != '\n' ){
                }	/* flush it */
                break;
        case 'a':       /* draw an arc */
                fprintf(stderr,"don't know how to draw arcs\n");
                for(i=0;i<6;i++)(void)getcoord();       /* flush it */
                break;
        case 'c':       /* draw a circle */
                fprintf(stderr,"don't know how to draw circles\n");
                for(i=0;i<3;i++)(void)getcoord();       /* flush it */
                break;
        case 'f':       /* change linestyle */
		/* should implement this */
                while((c=getchar()) != '\n');           /* flush it */
                break;
        default: 	/* print one error message */
		if(complaints == 0)fprintf(stderr,"unknown code = %c\n",code);
		complaints++;
    }
}

putimage(){
struct sgttyb ttystuff;
int i,j,n;
    /* if abnormal exit, we want to reset printer and tty */
    signal(SIGTERM,cleanup);
    signal(SIGINT,cleanup);
    signal(SIGHUP,cleanup);
    /* put tty in raw mode if stdout is a tty */
    if(ioctl(1,TIOCGETP,&ttystuff) == 0){
	ttystuff.sg_flags |= RAW;
	ioctl(1,TIOCSETP,&ttystuff);
    }
	
    printf("%cE",ESC);			/* reset */
    /**printf("%c&a+10R",ESC,10);**/		/* Move down 10 rows */
    printf("%c*rB",ESC);		/* end raster graphics */
    printf("%c*t%dR",ESC,density);	/* set graphics resolution */
    printf("%c*r%dS",ESC,8*density);	/* set raster width */
    printf("%c*r0A",ESC);		/* start raster graphics */
    fflush(stdout);
    for(j=rows-1;j>=0;j--){	/* loop over rows */
	/* begin graphics row of "cols" bits */
	printf("%c*b%dW",ESC,rowbytes);
	for(i=0;i<rowbytes;i++){
		putchar(image[j*rowbytes+i]);
	}
	fflush(stdout);
	/**if(j%8==0)sleep(1);**/
    }
    printf("%c*rB",ESC);		/* end raster graphics */
    printf("%cE",ESC);			/* reset */
    fflush(stdout);
    /* put tty back in cooked if stdout is a tty */
    if(ioctl(1,TIOCGETP,&ttystuff) == 0){
	ttystuff.sg_flags &= (~RAW);
	ioctl(1,TIOCSETP,&ttystuff);
    }
}

point(x,y) double x,y; {	/* insert a point in image */
    lightup(getrow(y),getcol(x));
}

line(x1,y1,x2,y2) double x1,y1,x2,y2; {	/* draw a line in image */
int i1,j1,i2,j2;	/* integers corresponding to arguments */
int i,j;	/* integers along line */
double x,y,dx,dy;	/* points along line, increments */
    i1=getcol(x1); j1 = getrow(y1);
    i2=getcol(x2); j2 = getrow(y2);

    lightup(j1,i1);	/* just in case line has ~0 length, plot a point */
    if(i2>i1)for(y=y1,dy=(y2-y1)/(i2-i1),i=i1;i<=i2;i++,y+=dy)
	lightup(getrow(y),i);
    else for(y=y1,dy=(y2-y1)/(i1-i2),i=i1;i>=i2;i--,y+=dy)
	lightup(getrow(y),i);
    if(j2>j1)for(x=x1,dx=(x2-x1)/(j2-j1),j=j1;j<=j2;j++,x+=dx)
	lightup(j,getcol(x));
    else for(x=x1,dx=(x2-x1)/(j1-j2),j=j1;j>=j2;j--,x+=dx)
	lightup(j,getcol(x));
}

getrow(y) double y; {	/* find the row index of a y coordinate */
    return((y-ymin)*yscale);
}
getcol(x) double x; {	/* find the column index of an x coordinate */
    return((x-xmin)*xscale);
}
lightup(row,col){    /* turn on a pixel */
register int i;
    i = row*rowbytes*8+col;	/* bit index */
    if(i>=rowbytes*8*rows){
	fprintf(stderr,"Oops, i = %d, row = %d, col = %d\n",i,row,col);
	return;
    }
    image[i>>3] |= mask[i&07];
#ifdef ALWAYSHEAVY
    if(density==300){  /* Make heavy lines even in high resolution mode */
        i++;
        if(col<cols)image[i>>3] |= mask[i&07];
        i+=rowbytes*8;
        if(col<cols&&row<rows)image[i>>3] |= mask[i&07];
        i--;
        if(row<rows)image[i>>3] |= mask[i&07];
    }
#endif
}

double getcoord(){
int c,d;
    c = getchar(); d = getchar();
    return( (double)((d<<8)|c) );
}

/* Clean up our mess on interrupt */
void cleanup(sig,code,scp) int sig,code; struct sigcontext *scp; {
int i;
struct sgttyb ttystuff;
    fprintf(stderr,"\ntjplot: interrupt exit, sig = %d, code = %d\n",sig,code);
    /* send "cols" bytes, to make sure we are not inside a graphics string */
    for(i=0;i<cols;i++)putchar('\0');
    /**printf("%c@",ESC);**/	/* Reset printer and form feed */
    /**/printf("%c@",ESC);
    fflush(stdout);
    /* put tty back in cooked if stdout is a tty */
    if(ioctl(1,TIOCGETP,&ttystuff) == 0){
	ttystuff.sg_flags &= (~RAW);
	ioctl(1,TIOCSETP,&ttystuff);
    }
    exit(2);	/* for lpr filter, 2 means job should be thrown away */
}
