/* graphics filter for HP XXXX laser printer                     *
*    150 dpi or 300 dpi resolution                          */

/* Use raw mode on tty, so can send eight bit characters */
#define ALWAYSHEAVY	/* take this out for fine lines in high res. mode */

#include <stdio.h>
#include <sgtty.h>
#include <signal.h>
#define ESC 27          /* ASCII escape   */
#define CR 13		/* Carriage return */
#define LF 10		/* Line feed */

double xmin,xmax,ymin,ymax,xscale,yscale,xdel,ydel;
/* boundaries of plot region, multipliers for converting to pixels, and
   distances between columns and rows */
int dflag=0;		/* 0 = low density, 1 = high density */
int density=150;		/* density in dots per inch */
int erase=0;		/* formfeed if erase */
int cols=1024,rows=1024; /* number of columns and rows, low density default */
char *image;		/* raster image */
static char mask[8] = {0200,0100,0040,0020,0010,0004,0002,0001 };
/* masks for picking single bits */
double getcoord();
char *malloc();
void cleanup();	/* interrupt handler */

main(argc,argv) char **argv; {

    mygetopt(argc,argv);	/* set options */
    setup();            /* initialize image, status variables */
    makeplot();         /* read input and make image            */
    putimage();         /* put result to printer                 */
}

mygetopt(argc,argv) char **argv; {
    if(argc > 1){
        if(strcmp("-x1024",argv[1])==0)dflag=0;	/* default */
        else if(strcmp("-x2048",argv[1])==0){
	    dflag=1;	/* high density */
	    rows=cols=2048;
	}
        else {
	    fprintf(stderr,"Unknown xwidth %s\n",argv[1]);
	    exit(2);
        }
    }
}

setup(){
    xmin = 0.0;         ymin = 0.0;     /* defaults */
    xmax = 1000.0;      ymax = 1000.0;
    xdel = 1.05*(xmax-xmin)/cols;   ydel = 1.05*(ymax-ymin)/rows;
    /* the 1.05 is to allow overlap on right */
    xscale = 1.0/xdel;          yscale = 1.0/ydel;
    image = malloc((cols*rows+7)/8);
    if( image==NULL ){fprintf(stderr,"Malloc failed\n"); exit(2); }
}

makeplot(){
int code,c,i,j,k,complaints;
double x,y,xcurrent,ycurrent,xnext,ynext;
    complaints=0;
    while( (code = getchar()) != EOF)switch(code){
        case 'e': erase = 1;    /* erase screen */
                break;
        case 's': xmin = getcoord();    /* set up plotting window */
                ymin = getcoord();
                xmax = getcoord();
                ymax = getcoord();
                xdel = 1.05*(xmax-xmin)/cols;	ydel = 1.05*(ymax-ymin)/rows;
                xscale = 1.0/xdel;              yscale = 1.0/ydel;
                break;
        case 'm':                       /* read in current point */
                xcurrent = getcoord();
                ycurrent = getcoord();
                break;
        case 'p':                       /* plot a point */
                xcurrent = getcoord();
                ycurrent = getcoord();
                point(xcurrent,ycurrent);
                break;
        case 'l':                       /* plot a line  */
                xcurrent = getcoord();
                ycurrent = getcoord();
                        /* continue on into case 'n' */
        case 'n':                       /* line from current to next*/
                xnext = getcoord();
                ynext = getcoord();
		line(xcurrent,ycurrent,xnext,ynext);
                xcurrent = xnext;       ycurrent = ynext;
                break;
        case 't':       /* put a label in the graph */
                fprintf(stderr,"don't know how to make labels\n");
                while( (c=getchar()) != '\n' ){
                }	/* flush it */
                break;
        case 'a':       /* draw an arc */
                fprintf(stderr,"don't know how to draw arcs\n");
                for(i=0;i<6;i++)(void)getcoord();       /* flush it */
                break;
        case 'c':       /* draw a circle */
                fprintf(stderr,"don't know how to draw circles\n");
                for(i=0;i<3;i++)(void)getcoord();       /* flush it */
                break;
        case 'f':       /* change linestyle */
		/* should implement this */
                while((c=getchar()) != '\n');           /* flush it */
                break;
        default: 	/* print one error message */
		if(complaints == 0)fprintf(stderr,"unknown code = %c\n",code);
		complaints++;
    }
}

putimage(){
struct sgttyb ttystuff;
int i,j,n;
    /* if abnormal exit, we want to reset printer and tty */
    signal(SIGTERM,cleanup);
    signal(SIGINT,cleanup);
    signal(SIGHUP,cleanup);
    /* put tty in raw mode if stdout is a tty */
    if(ioctl(1,TIOCGETP,&ttystuff) == 0){
	ttystuff.sg_flags |= RAW;
	ioctl(1,TIOCSETP,&ttystuff);
    }
	
    printf("%cE",ESC);	/* Set line spacing */
    if(dflag==0){
	printf("%c*t%dR",ESC,150);	/* Set low density */
        printf("%c*p%dx%dY",ESC,150,400);	/*Initial x and y coordinates */
    }
    else {
	printf("%c*t%dR",ESC,300);	/* Set high density */
        printf("%c*p%dx%dY",ESC,300,800);	/*Initial x and y coordinates */
    }
    printf("%c*r1A",ESC);		/* Start graphics at cursor position */
    n = cols >> 3  ;	/* number of cols of bytes */
    for(j=rows-1;j>=0;j--){	/* loop over rows */
	/* begin graphics row of "n" bytes */
	printf("%c*b%dW",ESC,n);
	for(i=0;i<n;i++){
		putchar(image[j*n+i]);
	}
    }
    printf("%c*rB",ESC);	/* Reset printer and form feed */
    fflush(stdout);
    /* put tty back in cooked if stdout is a tty */
    if(ioctl(1,TIOCGETP,&ttystuff) == 0){
	ttystuff.sg_flags &= (~RAW);
	ioctl(1,TIOCSETP,&ttystuff);
    }
}

point(x,y) double x,y; {	/* insert a point in image */
    lightup(getrow(y),getcol(x));
}

line(x1,y1,x2,y2) double x1,y1,x2,y2; {	/* draw a line in image */
int i1,j1,i2,j2;	/* integers corresponding to arguments */
int i,j;	/* integers along line */
double x,y,dx,dy;	/* points along line, increments */
    i1=getcol(x1); j1 = getrow(y1);
    i2=getcol(x2); j2 = getrow(y2);

    lightup(j1,i1);	/* just in case line has ~0 length, plot a point */
    if(i2>i1)for(y=y1,dy=(y2-y1)/(i2-i1),i=i1;i<=i2;i++,y+=dy)
	lightup(getrow(y),i);
    else for(y=y1,dy=(y2-y1)/(i1-i2),i=i1;i>=i2;i--,y+=dy)
	lightup(getrow(y),i);
    if(j2>j1)for(x=x1,dx=(x2-x1)/(j2-j1),j=j1;j<=j2;j++,x+=dx)
	lightup(j,getcol(x));
    else for(x=x1,dx=(x2-x1)/(j1-j2),j=j1;j>=j2;j--,x+=dx)
	lightup(j,getcol(x));
}

getrow(y) double y; {	/* find the row index of a y coordinate */
    return((y-ymin)*yscale);
}
getcol(x) double x; {	/* find the column index of an x coordinate */
    return((x-xmin)*xscale);
}

lightup(row,col){    /* turn on a pixel */
register int i;
    i = col+row*cols;	/* bit index */
    if(i>=cols*rows){
	/*fprintf(stderr,"Oops, i = %d, row = %d, col = %d\n",i,row,col);*/
	return;
    }
    image[i>>3] |= mask[i&07];
#ifdef ALWAYSHEAVY
    if(dflag==1){  /* Make heavy lines even in high resolution mode */
        i++;
        if(i<cols*rows)image[i>>3] |= mask[i&07];
	i+=cols;
        if(i<cols*rows)image[i>>3] |= mask[i&07];
	i--;
        if(i<cols*rows)image[i>>3] |= mask[i&07];
    }
#endif
}

double getcoord(){
int c,d;
    c = getchar(); d = getchar();
    return( (double)((d<<8)|c) );
}

/* Clean up our mess on interrupt */
void cleanup(sig,code,scp) int sig,code; struct sigcontext *scp; {
int i;
struct sgttyb ttystuff;
    fprintf(stderr,"\nhplaser: interrupt exit, sig = %d, code = %d\n",sig,code);
    /* send "cols/8" bytes, to make sure we are not inside a graphics string */
    for(i=0;i<cols/8;i++)putchar('\0');
    printf("%c*rB",ESC);	/* Reset printer and form feed */
    fflush(stdout);
    /* put tty back in cooked if stdout is a tty */
    if(ioctl(1,TIOCGETP,&ttystuff) == 0){
	ttystuff.sg_flags &= (~RAW);
	ioctl(1,TIOCSETP,&ttystuff);
    }
    exit(2);	/* for lpr filter, 2 means job should be thrown away */
}
