/* graphics filter for z19 terminal                     *
*  240 x 72 resolution                                  *
*  each character is divided into 9 cells (3x3)         *
*  cells are "lit" when a line or point in graph falls  *
* nearby. After all cells are lit, a character is       *
* selected which best represents the pattern            *
* The character table is in sym[].                      *
* Octal indices and masks are convenient for 3x3        *
* cells.                                                *
* Cells in the character are denoted by bits in the     *
* index as follows:                                     *
* lower left = 0001, lower center = 0002,lower right = 0004     *
* center left = 0010, center**2 = 0020, center right = 0040,    *
* upper left = 0100, upper center = 0200, upper right = 0400.   */
#define LMAX 1000               /* maximum number of labels     */
#include <stdio.h>
#define REV 0x80        /* mask for bit 7               */
#define ESC 27          /* ASCII excape                 */
#define clear()         {putchar(ESC);putchar('E');}
#define graph_on()      {putchar(ESC);putchar('F');}
#define graph_off()     {putchar(ESC);putchar('G');}
#define reverse_on()    {putchar(ESC);putchar('p');}
#define reverse_off()   {putchar(ESC);putchar('q');}
#define putcurse(i,j)   {putchar(ESC);putchar('Y');\
                putchar(55-(i));putchar(32+(j));}
short image[24][80];    /* map of screen, each character*
                        *  holds code for 9 dots        */
char sym[512];  /* graphics symbols used                */
                        /* high order bit marks reverse video */
int reverse;            /* remember if we are in reverse video */
struct { double x,y;                    /* coordinates of label */
        char text[50];} label[LMAX];    /* text of label        */
int ilabel;                             /* index in label[]     */
double xmin,xmax,ymin,ymax,xscale,yscale,xdel,ydel;
double getcoord();
int erase;
int mask[9];            /* mask for lighting up bits */
main(){
    setup();            /* initialize image, status variables */
    makeplot();         /* read input and make image            */
    putimage();         /* put result on screen                 */
    putlabels();        /* add the labels                       */
    putcurse(0,0);      /* cursor to lower left                 */
}
setup(){
    int i,j;
    for(i=0;i<24;i++)for(j=0;j<80;j++)image[i][j] = 0;
    erase = 0;
    xmin = 0.0;         ymin = 0.0;     /* defaults */
    xmax = 1000.0;      ymax = 1000.0;
    xdel = (xmax-xmin)/240.0;   ydel = (ymax-ymin)/72.0;
    xscale = 1.0/xdel;          yscale = 1.0/ydel;
    for(mask[0] = i = 1;i<9;i++)mask[i] = 2*mask[i-1];
    /*for(i=0;i<512;i++)sym[i] = 103; *//* '+/-' = default */
    for(i=0;i<512;i++)sym[i] = 32 | REV;
    /*sym[0777] = sym[0776] = sym[0773] = sym[0677] = sym[0377] = 32 | REV;
    sym[0767] = sym[0737] = 32 | REV;
    sym[0712] = sym[0742] = sym[0217] = sym[0247] = 32 | REV;
    sym[0732] = sym[0762] = sym[0237] = sym[0267] = 32 | REV;
    sym[0326] = sym[0623] = sym[0336] = sym[0633] = 32 | REV;
    sym[0663] = sym[0366] = sym[0754] = sym[0751] = 32 | REV;
    sym[0772] = sym[0277] = sym[0457] = sym[0157] = 32 | REV;
    sym[0637] = sym[0367] = sym[0736] = sym[0763] = 32 | REV;
    sym[0726] = sym[0723] = sym[0627] = sym[0327] = 32 | REV;*/
    sym[0020] = sym[0040] = sym[0010] = sym[0200] = sym[0002] = 94;
    sym[0757] = 94 | REV;
    sym[0764] = sym[0744] = sym[0704] = sym[0224] = sym[0564] = 95;
    sym[0724] = sym[0734] = sym[0774] = sym[0544] = sym[0364] = 95;
    sym[0137] = sym[0117] = sym[0107] = sym[0122] = sym[0135] = 95 | REV;
    sym[0127] = sym[0167] = sym[0177] = sym[0115] = 95 | REV;
    sym[0202] = sym[0220] = sym[0022] = sym[0222] = 96;
    sym[0322] = sym[0622] = sym[0223] = sym[0226] = 96;
    sym[0722] = sym[0227] = 96;
    sym[0555] = sym[0550] = sym[0055] = 96 | REV;
    sym[0070] = sym[0050] = sym[0030] = sym[0060] = 97;
    sym[0170] = sym[0470] = sym[0071] = sym[0074] = 97;
    sym[0061] = sym[0160] = sym[0034] = sym[0430] = 97;
    sym[0707] = sym[0347] = sym[0617] = sym[0743] = sym[0716] = 97 | REV;
    sym[0747] = sym[0717] = 97 | REV;
    sym[0307] = sym[0607] = sym[0703] = sym[0706] = 97 | REV;
    sym[0303] = sym[0606] = sym[0306] = sym[0603] = 97 | REV;
    sym[0272] = sym[0236] = sym[0263] = sym[0632] = sym[0362] = 98;
    sym[0276] = sym[0273] = sym[0672] = sym[0372] = 98;
    sym[0505] = 98 | REV;
    sym[0032] = sym[0012] = sym[0132] = sym[036] = 99;
    sym[0745] = 99 | REV;
    sym[0230] = sym[0210] = sym[0630] = sym[0231] = 100;
    sym[0547] = 100 | REV;
    sym[0260] = sym[0240] = sym[0360] = sym[0264] = 101;
    sym[0517] = 101 | REV;
    sym[0062] = sym[0042] = sym[0462] = sym[0063] = 102;
    sym[0715] = 102 | REV;
    sym[0066] = sym[0046] = sym[0004] = 108;
    sym[0033] = sym[0013] = sym[0001] = 109;
    sym[0330] = sym[0310] = sym[0100] = 110;
    sym[0660] = sym[0640] = sym[0400] = 111;
    sym[0760] = sym[0730] = sym[0670] = sym[0370] = 112;
    sym[0750] = sym[0570] = 112;
    sym[0067] = sym[0037] = sym[0076] = sym[0073] = 112 | REV;
    sym[0075] = sym[0057] = 112 | REV;
    sym[0666] = sym[0664] = sym[0466] = sym[0646] = 113;
    sym[0442] = sym[0424] = sym[0244] = sym[0474] = 113;
    sym[0644] = sym[0464] = sym[0446] = 113;
    sym[0746] = sym[0647] = sym[0266] = sym[0662] = 113;
    sym[0333] = sym[0133] = sym[0313] = sym[0331] = 113 | REV;
    sym[0112] = sym[0121] = sym[0211] = sym[0171] = 113 | REV;
    sym[0113] = sym[0131] = sym[0311] = 113 | REV;
    sym[0713] = sym[0317] = sym[0332] = sym[0233] = 113 | REV;
    sym[0731] = sym[0711] = sym[0221] = sym[0531] =  sym[0721] =114;
    sym[0701] = sym[0761] = sym[0771] = sym[0511] = sym[0631] = 114;
    sym[0467] = sym[0447] = sym[0422] = sym[0465] = sym[0427] = 114 | REV;
    sym[0407] = sym[0437] = sym[0477] = sym[0445] = 114 | REV;
    sym[0072] = 115;    sym[0705] = 115 | REV;
    sym[0232] = 116;    sym[0545] = 116 | REV;
    sym[0270] = 117;    sym[0507] = 117 | REV;
    sym[0262] = 118;    sym[0515] = 118 | REV;
    sym[0525] = 119;    sym[0252] = 119 | REV;
    sym[0421] = sym[0423] = sym[0431] = sym[0461] = sym[0621] = 120;
    sym[0021] = sym[0420] = sym[0443] = sym[0611] = sym[0471] = 120;
    sym[0463] = sym[0631] = sym[0463] = 120;
    sym[0356] = 120 | REV;
    sym[0124] = sym[0324] = sym[0134] = sym[0164] = sym[0126] = 121;
    sym[0120] = sym[0024] = sym[0116] = sym[0344] = sym[0174] = 121;
    sym[0136] = sym[0364] = sym[0136] = 121;
    sym[0653] = 121 | REV;
    sym[0700] = sym[0500] = sym[0300] = sym[0600] = 122;
    sym[0610] = sym[0340] = sym[0710] = sym[0740] = sym[0720] = 122;
    sym[0077] = 122 | REV;
    sym[0007] = sym[0005] = sym[0003] = sym[0006] = 123;
    sym[0016] = sym[0043] = sym[0017] = sym[0047] = sym[0027] = 123;
    sym[0770] = 123 | REV;
    sym[0101] = sym[0110] = sym[0011] = sym[0111] = 124;
    sym[0404] = sym[0440] = sym[0044] = sym[0444] = 125;
}
makeplot(){
int code,c,i,j,k;
double x,y,xcurrent,ycurrent,xnext,ynext;
    while( (code = getchar()) != EOF)switch(code){
        case 'e': erase = 1;    /* erase screen */
                break;
        case 's': xmin = getcoord();    /* set up plotting window */
                ymin = getcoord();
                xmax = getcoord();
                ymax = getcoord();
                xdel = (xmax-xmin)/240.0;       ydel = (ymax-ymin)/72.0;
                xscale = 1.0/xdel;              yscale = 1.0/ydel;
                break;
        case 'm':                       /* read in current point */
                xcurrent = getcoord();
                ycurrent = getcoord();
                break;
        case 'p':                       /* plot a point */
                xcurrent = getcoord();
                ycurrent = getcoord();
                lightup(xcurrent,ycurrent);
                break;
        case 'l':                       /* plot a line  */
                xcurrent = getcoord();
                ycurrent = getcoord();
                        /* continue on into case 'n' */
        case 'n':                       /* line from current to next*/
                xnext = getcoord();
                ynext = getcoord();
                lightup(xcurrent,ycurrent);
                lightup(xnext,ynext);
                if(xnext > xcurrent + xdel)
                    for(x=xcurrent+xdel;x<xnext;x+=xdel){
                        y = ycurrent+(ynext-ycurrent)*
                            (x-xcurrent)/(xnext-xcurrent);
                        lightup(x,y);
                    }
                else for(x=xcurrent-xdel;x>xnext;x-=xdel){
                        y = ycurrent+(ynext-ycurrent)*
                            (x-xcurrent)/(xnext-xcurrent);
                        lightup(x,y);
                    }
                if(ynext > ycurrent + ydel)
                    for(y=ycurrent+ydel;y<ynext;y+=ydel){
                        x = xcurrent+(xnext-xcurrent)*
                            (y-ycurrent)/(ynext-ycurrent);
                        lightup(x,y);
                    }
                else for(y=ycurrent-ydel;y>ynext;y-=ydel){
                        x = xcurrent+(xnext-xcurrent)*
                            (y-ycurrent)/(ynext-ycurrent);
                        lightup(x,y);
                    }
                xcurrent = xnext;       ycurrent = ynext;
                break;
        case 't':       /* put a label in the graph */
                if( ilabel >= LMAX){
                        fprintf(stderr,"too many labels\n");
                        break;
                }
                label[ilabel].x = xcurrent;
                label[ilabel].y = ycurrent;
                i=0;
                while( (c=getchar()) != '\n' ){
                        if ( i < 49 ) label[ilabel].text[i++] = c;
                        xcurrent += xdel;       /* cursor moves */
                }
                label[ilabel].text[i] = '\0';   /* terminate string */
                ilabel++;
                break;
        case 'a':       /* draw an arc */
                fprintf(stderr,"don't know how to draw arcs\n");
                for(i=0;i<6;i++)(void)getcoord();       /* flush it */
                break;
        case 'c':       /* draw a circle */
                fprintf(stderr,"don't know how to draw circles\n");
                for(i=0;i<3;i++)(void)getcoord();       /* flush it */
                break;
        case 'f':       /* change linestyle */
                /* on a z19 you've got to be kidding */
                while((c=getchar()) != '\n');           /* flush it */
                break;
        default: fprintf(stderr,"unknown code = %c\n",code);
    }
}
putimage(){
int i,j;
int c;
    if(erase) clear();
    reverse_off();reverse=0;    /* make sure reverse video is off */
    graph_on();         /* turn on graphics mode        */
    for(i=23;i>=0;i--)for(j=0;j<80;j++)if( (c=image[i][j]) != 0 ){
        switch(c){      /* special handling for nasty cases     */
                        /* like only corner cells lit           */
/* if only one corner cell is lit, put blank if a neighboring cell is lit */
            case 0001:  /* lower left only */
if( (j>0 && (image[i][j-1] & 0004)) || (i>0 && (image[i-1][j] & 0100)) )
                {image[i][j]=0; goto skip;}; break;
            case 0004:  /* lower right only */
if( (j<79 && (image[i][j+1] & 0001)) || (i>0 && (image[i-1][j] & 0400)) )
                {image[i][j]=0; goto skip;}; break;
            case 0100:  /* upper left only */
if( (j>0 && (image[i][j-1] & 0400)) || (i<23 && (image[i+1][j] & 0001)) )
                {image[i][j]=0; goto skip;}; break;
            case 0400:  /* upper right only */
if( (j<79 && (image[i][j+1] & 0100)) ||(i<23 && (image[i+1][j] & 0004)) )
                {image[i][j]=0; goto skip;}; break;
/* if only one side cell is lit, put blank if a neighboring cell is lit */
            case 0002:
if( i>0 && (image[i-1][j] & 0200) ) {image[i][j]=0; goto skip;}; break;
            case 0040:
if( j<79 && (image[i][j+1] & 0010) ) {image[i][j]=0; goto skip;}; break;
            case 0010:
if( j>0 && (image[i][j-1] & 0040) ) {image[i][j]=0; goto skip;}; break;
            case 0200:
if( i<23 && (image[i+1][j] & 0002) ) {image[i][j]=0; goto skip;}; break;
/* if upper or lower bar is lit and opposite corner */
/* clobber if it has neighbors */
/**         case 0701:
if( (j>0 && (image[i][j-1] & 0004)) || (i>0 && (image[i-1][j] & 0100)) )
                c = 0700; break;
            case 0704:
if( (j<79 && (image[i][j+1] & 0001)) || (i>0 && (image[i-1][j] & 0400)) )
                c = 0700; break;
            case 0107:
if( (j>0 && (image[i][j-1] & 0400)) || (i<23 && (image[i+1][j] & 0001)) )
                c = 0007; break;
            case 0407:
if( (j<79 && (image[i][j+1] & 0100)) ||(i<23 && (image[i+1][j] & 0004)) )
                c = 0007; break;**/
        }
        if(  (sym[c] & REV) && !reverse){reverse_on(); reverse = 1;}
        else if( !(sym[c] & REV) && reverse){reverse_off();reverse = 0;}
        putcurse(i,j);  /* move cursor */
        putchar( (sym[c] & 0x7f) );
skip:;
    }
    reverse_off(); reverse=0;
    graph_off();
}
putlabels(){
int i,j,ix,iy;
/** temporary kludge: put only three labels (x and y scales)    **/
    for(i=0;i<ilabel && i < 3;i++){
        ix = (int)((label[i].x - xmin)*xscale);
        iy = (int)((label[i].y - ymin)*yscale);
        if(iy > 0 )iy--;        /* kludge. I don't understand it */
        putcurse(iy,ix);
        for(j=0;label[i].text[j] != NULL;j++)
            if(ix++ <= 79)putchar(label[i].text[j]);
    }
}
lightup(xx,yy)double xx,yy;{    /* put a point at or near (xx,yy) */
int i,j,select;
    if( xx >= xmin && xx <= xmax && yy >= ymin && yy <= ymax){
        i = (int)((yy-ymin)*yscale);
        if(i & 1)select = 2; else select = 0;
        switch( i%3){
            case 2: select = 6; break;
            case 1: select = 3; break;
            case 0: select = 0; break;
        }
        i /= 3;
        j = (int)((xx-xmin)*xscale);
        select += j%3;
        j /= 3;
        image[i][j] |= mask[select];
    }
}
double getcoord(){
int c,d;
   c = getchar();
    d = getchar();
    return( (double)((d<<8)|c) );
}
