/* UNIX graphics filter for SUN 2 monitor 			*/
/* 6/18/84 Doug Toussaint					*/
/* not neat, but compatible with old UNIX graphics		*/
/* to compile:							*/
/* cc -o psun -O -fsingle psun.c -lcore -lsunwindow -lpixrect -lm  */

#define LINEWIDTH 0.2	/* in units of 1% of NDC space */
#define POINTSIZE 0.5	/* This seems to do nothing    */
float linewidth;
float scale;		/* overall scale factor */


#include <stdio.h>
#include <usercore.h>
float getcoord();
int bw2dd(),pixwindd();	/* sun 2 bw monitor or graphics window */
struct vwsurf raw_surf=DEFAULT_VWSURF(bw2dd);
struct vwsurf win_surf=DEFAULT_VWSURF(pixwindd);
struct vwsurf *surf_pt;
char *getenv();

main(argc,argv) int argc; char **argv; {
int code,i,j;
float x,y,xmin,xmax,ymin,ymax,xcurrent,ycurrent,xscale,yscale;
float nd_height,nd_width,nd_size;
char label[128];
    linewidth=LINEWIDTH;
    scale=1.0;

   /* read options */
   for(i=1;i<argc;i++){
       if(strcmp(argv[i],"-w")==0){
	   if(i==argc-1) fprintf(stderr,"Missing psun linewidth option\n");
	   else if(sscanf(argv[++i],"%e",&x) == 1){
	       linewidth=x;
	   }
	   else fprintf(stderr,"Bad psun linewidth option %s\n",argv[i]);
       }
       else if(strcmp(argv[i],"-s")==0){
	   if(i==argc-1) fprintf(stderr,"Missing psun scale option\n");
	   else if(sscanf(argv[++i],"%e",&x) == 1) scale=x;
	   else fprintf(stderr,"Bad psun scale option %s\n",argv[i]);
       }
       else fprintf(stderr,"Illegal psun option %s\n",argv[i]);
   }

    /* decide whether we are going to a raw screen or a window */
    if(getenv("WINDOW_ME") != NULL)surf_pt = &win_surf;
    else surf_pt = &raw_surf;

    if(initialize_core(BASIC,SYNCHRONOUS,TWOD))exit(1);
    if(initialize_view_surface(surf_pt,FALSE))exit(2);
    if(select_view_surface(surf_pt))exit(3);
    if(initialize_device(KEYBOARD,1))exit(4);

    /* find out what the device coordinates are */
    inquire_ndc_space_2(&nd_width,&nd_height);
    /* want a square area, choose smaller */
    nd_size = nd_width > nd_height ? nd_height : nd_width;
    xscale = nd_width/nd_size;	yscale = nd_height/nd_size;

    /* now begin translating input file to SunCore calls */
    while( (code = getchar()) != EOF)switch(code){
        case 'e':
		new_frame();			/* erase screen */
                break;
        case 's': xmin = xscale*getcoord();    /* set up plotting window */
                ymin = yscale*getcoord();
                xmax = xscale*getcoord();
                ymax = yscale*getcoord();
	        /* now rescale plot window, keeping it centered */
	        x = 0.5*(xmax+xmin);
	        xmin = x-(x-xmin)/scale;
	        xmax = x+(xmax-x)/scale;
	        x = 0.5*(ymax+ymin);
	        ymin = x-(x-ymin)/scale;
	        ymax = x+(ymax-x)/scale;
	        /* end rescaling */
		set_window(xmin,xmax,ymin,ymax);
		set_charsize(xmax/80,ymax/34);	/* string characters */
			/* assuming full screen here */
		set_linewidth(linewidth);
		create_retained_segment(1);
                break;
        case 'm':                       /* read in current point */
                xcurrent = getcoord();
                ycurrent = getcoord();
		move_abs_2(xcurrent,ycurrent);
                break;
        case 'p':                       /* plot a point */
                xcurrent = getcoord();
                ycurrent = getcoord();
		move_abs_2(xcurrent,ycurrent);
		line_abs_2(xcurrent,ycurrent);
                break;
        case 'l':                       /* plot a line  */
                xcurrent = getcoord();
                ycurrent = getcoord();
		move_abs_2(xcurrent,ycurrent);
                        /* continue on into case 'n' */
        case 'n':                       /* line from current to next*/
                xcurrent = getcoord();
                ycurrent = getcoord();
		line_abs_2(xcurrent,ycurrent);
                break;
        case 't':       /* put a label in the graph */
		text(gets(label));
		inquire_text_extent_2(label,&x,&y);
		xcurrent += x;	/* graph() assumes cursor will move */
		move_abs_2(xcurrent,ycurrent);
                break;
        case 'a':       /* draw an arc */
                fprintf(stderr,"don't know how to draw arcs\n");
                for(i=0;i<6;i++)(void)getcoord();       /* flush it */
                break;
        case 'c':       /* draw a circle */
                fprintf(stderr,"don't know how to draw circles\n");
                for(i=0;i<3;i++)(void)getcoord();       /* flush it */
                break;
        case 'f':       /* change linestyle */
		gets(label);
		if(strcmp(label,"solid")==0)set_linestyle(SOLID);
		else if(strcmp(label,"dotted")==0)set_linestyle(DOTTED);
		else if(strcmp(label,"shortdashed")==0)set_linestyle(DASHED);
		else if(strcmp(label,"longdashed")==0)set_linestyle(DASHED);
		else if(strcmp(label,"dotdashed")==0)set_linestyle(DOTDASHED);
		break;
        default: fprintf(stderr,"unknown code = %c\n",code);
    }

    close_retained_segment(1);
    /***/while(1)sleep(60);/***/
    /**j=await_keyboard(1000000000,1,label,&i);**/
	/* This seems to wait until interrupted by ^C ??*/
    terminate_device(KEYBOARD,1);
    deselect_view_surface(surf_pt);
    terminate_core();

}
 
float getcoord(){
int c,d;
   c = getchar();
    d = getchar();
    return( (float)((d<<8)|c) );
}
