/* UNIX graphics filter for QUIC plotting language of Talaris and QMS laser
   printers.   June, 1986  S. Gottlieb					   */

#include <stdio.h>
#define XDOTS 2400	/* 8 inches at 300 dots per inch  */
#define YDOTS 2400	/* ditto  */

#define LANDSCAPE 73
#define PORTRAIT 74
int orientation;	/* PORTRAIT or LANDSCAPE */
int linewidth;		/* in pixels, odd number from 3 to 31 */
double scale;		/* overall scale factor */

main(argc,argv) int argc ; char **argv;   {
int code,xdev,ydev;
int i,j,ll;
int ersct = 0 ;	/* erase count	*/ 
double getcoord();
double x,xmin,xmax,ymin,ymax,xscale,yscale,xcurrent,ycurrent;
char label[128];
   xmin=0.0;		ymin=0.0;	/* defaults */
   xmax=1000.0;		ymax=1000.0;
   xscale=XDOTS/xmax;	yscale=YDOTS/ymax;
   orientation = LANDSCAPE;
   linewidth = 5;
   scale = 1.0;

   /* read options */
   for(i=1;i<argc;i++){
       if(strcmp(argv[i],"-p")==0){
	   orientation = PORTRAIT;
       }
       else if(strcmp(argv[i],"-w")==0){
	   if(i==argc-1) fprintf(stderr,"Missing ptal width option\n");
	   else if(sscanf(argv[++i],"%d",&j) == 1){
	       if(j<3 || j>31 || j%2 != 1)
		   fprintf(stderr,"Bad ptal width option %s\n",argv[i]);
	       else linewidth=j;
	   }
	   else fprintf(stderr,"Bad ptal width option %s\n",argv[i]);
       }
       else if(strcmp(argv[i],"-s")==0){
	   if(i==argc-1) fprintf(stderr,"Missing ptal scale option\n");
	   else if(sscanf(argv[++i],"%le",&x) == 1) scale=x;
	   else fprintf(stderr,"Bad ptal scale option %s\n",argv[i]);
       }
       else fprintf(stderr,"Illegal ptal option %s\n",argv[i]);
   }

   /* prepare the laser printer */
   setup();
   ll=0;

   /* now begin translating input file to QUIC commands */
   while( (code=getchar()) != EOF) switch(code)   {
      case 'e':				/* erase, start new page */
	      if (ersct++ != 0) printf("^,");		/* plot */
	      break;
      case 's':				/* set up plotting window */
	      xmin=getcoord();
	      ymin=getcoord();
	      xmax=getcoord();
	      ymax=getcoord();
	      /* now rescale plot window, keeping it centered */
	      x = 0.5*(xmax+xmin);
	      xmin = x-(x-xmin)/scale;
	      xmax = x+(xmax-x)/scale;
	      x = 0.5*(ymax+ymin);
	      ymin = x-(x-ymin)/scale;
	      ymax = x+(ymax-x)/scale;
	      /* end rescaling */
	      xscale=XDOTS/(xmax-xmin);
	      yscale=YDOTS/(ymax-ymin);
	      break;
      case 'm':				/* move to a point */
	      xcurrent=getcoord();
	      ycurrent=getcoord();
	      xdev=(xcurrent-xmin)*xscale;
	      ydev=(ymax-ycurrent)*yscale;	/* lefthanded QMS coords. */
	      ll=cr(ll,11);
	      printf("^U%d:%d",xdev,ydev);
	      break;
      case 'p':					/* plot a point */
	      xcurrent=getcoord();
	      ycurrent=getcoord();
	      xdev=(xcurrent-xmin)*xscale;
	      ydev=(ymax-ycurrent)*yscale;	/* lefthanded QMS coords. */
	      ll=cr(ll,22);
	      printf("^U%d:%d^D%d:%d",xdev,ydev,xdev,ydev);
	      break;
      case 'l':					/* plot a line */
	      xcurrent=getcoord();
	      ycurrent=getcoord();
	      xdev=(xcurrent-xmin)*xscale;
	      ydev=(ymax-ycurrent)*yscale;	/* lefthanded QMS coords. */
	      ll=cr(ll,11);
	      printf("^U%d:%d",xdev,ydev);
	      					/* continue to next case */
      case 'n':					/* line from current to next */
	      xcurrent=getcoord();
	      ycurrent=getcoord();
	      xdev=(xcurrent-xmin)*xscale;
	      ydev=(ymax-ycurrent)*yscale;	/* lefthanded QMS coords. */
	      ll=cr(ll,11);
	      printf("^D%d:%d",xdev,ydev);
	      break;
      case 't':					/* put a label on graph */
	      gets(label);
	      printf("\n^O^IGE%s^F^IGV\n",label);
	      ll=0;
	      break;
      case 'a':					/* draw an arc */
	      fprintf(stderr,"don't know how to draw an arc yet\n");
	      for(i=0;i<6;i++) (void)getcoord();
	      break;
      case 'c':					/* draw an circle */
	      fprintf(stderr,"don't know how to draw an circle yet\n");
	      for(i=0;i<3;i++) (void)getcoord();
	      break;
      case 'f':					/* change linestyle */
	      gets(label);
	      if(strcmp(label,"solid") == 0 )
		 {ll=cr(ll,3);
		 printf("^V0");}
	      else if (strcmp(label,"dotted") == 0 ) 
		 {ll=cr(ll,3);
		 printf("^V2");}
	      else if (strcmp(label,"shortdashed") == 0 ) 
		 {ll=cr(ll,3);
		 printf("^V3");}
	      else if (strcmp(label,"longdashed") == 0 ) 
		 {ll=cr(ll,3);
		 printf("^V6");}
	      else if (strcmp(label,"dotdashed") == 0 ) 
		 {ll=cr(ll,3);
		 printf("^V7");}
	      break;
      default: fprintf(stderr,"unknown code = %c\n",code);
	      break;
      }
      /* terminate the plot */
      printf("^,^IGE^O\n^-^PN^-\n");
}

double getcoord(){
int c,d;
   c=getchar();
   d=getchar();
   return( (double)((d<<8)|c) );
}

cr(ll,i) int ll,i;  {
   if( (ll+=i) <= 80 ) return(ll);
   printf("\n");
   return(i);
}

setup() {
   printf("\n^PY^-\n");		/* enter QUIC mode, must be on separate line */
   if(orientation==LANDSCAPE)
       printf("^IOL\n");		/* landscape mode */
   else
       printf("^IOP\n");		/* portrait mode */
   printf("^ISYNTAX00110\n");	/* units are dots, right justified */
   printf("^IFXX0X\n");		/* something about paper source */
   if(orientation==LANDSCAPE)
       printf("^IMV00752475^IMH04502850^F^IGV^PW%02d\n",linewidth);
   else
       printf("^IMV04502850^IMH00752475^F^IGV^PW%02d\n",linewidth);
	/* set up margins for 8 inch square with equal borders, turn on free
	format, vector graphics and set pen width to 'linewidth' pixels */
}

