/* UNIX graphics filter for Visual 550 terminal			*/
/* 1/31/85 Doug Toussaint					*/
/* Can be used on almost any tektroniks compatible device, sometimes
   with addition of leading or trailing character sequences to
   put into or take out of graphics mode */

#include <stdio.h>
#define ESC 27          /* ASCII escape                 */
#define FF 014		/* Formfeed */
#define POINT 034	/* enter point plot mode */
#define VECTOR 035	/* enter vector mode */
#define ALPHA 037	/* enter alphagraphics mode */
#define XMIN 0		/* square plotting window in device coordinates */
#define XMAX 750
#define YMIN 0
#define YMAX 750
#define XDOTS (XMAX-XMIN)
#define YDOTS (YMAX-YMIN)
double getcoord();

main(){
int code,c,i,xdev,ydev,mode;
double xmin,xmax,ymin,ymax,xscale,yscale,xdel,ydel,xcurrent,ycurrent;
char label[128];
    xmin = 0.0;         ymin = 0.0;     /* defaults */
    xmax = 1000.0;      ymax = 1000.0;
    xdel = (xmax-xmin)/XDOTS;   ydel = (ymax-ymin)/YDOTS;
    xscale = 1.0/xdel;          yscale = 1.0/ydel;

    /* turn on plot mode  */
    putchar(ALPHA); mode=ALPHA;

    /* now begin translating input file to Vi550 commands */
    while( (code = getchar()) != EOF)switch(code){
        case 'e':			/* erase screen */
		putchar(ESC);putchar(FF);
		mode = ALPHA;
		/* escape formfeed clears screen, leaves us in ALPHA mode */
                break;
        case 's': xmin = getcoord();    /* set up plotting window */
                ymin = getcoord();
                xmax = getcoord();
                ymax = getcoord();
                xdel = (xmax-xmin)/XDOTS;       ydel = (ymax-ymin)/YDOTS;
                xscale = 1.0/xdel;              yscale = 1.0/ydel;
                break;
        case 'm':                       /* read in current point */
                xcurrent = getcoord();
                ycurrent = getcoord();
		xdev = (xcurrent-xmin)*xscale+XMIN;
		ydev = (ycurrent-ymin)*yscale+YMIN;
		putchar(VECTOR);mode = VECTOR;
		putcoord(xdev,ydev);
                break;
        case 'p':                       /* plot a point */
                xcurrent = getcoord();
                ycurrent = getcoord();
		xdev = (xcurrent-xmin)*xscale+XMIN;
		ydev = (ycurrent-ymin)*yscale+YMIN;
		if(mode != POINT) {putchar(POINT);mode = POINT;}
		putcoord(xdev,ydev);
                break;
        case 'l':                       /* plot a line  */
                xcurrent = getcoord();
                ycurrent = getcoord();
		xdev = (xcurrent-xmin)*xscale+XMIN;
		ydev = (ycurrent-ymin)*yscale+YMIN;
		putchar(VECTOR); mode = VECTOR;
		putcoord(xdev,ydev);
                        /* continue on into case 'n' */
        case 'n':                       /* line from current to next*/
                xcurrent = getcoord();
                ycurrent = getcoord();
		xdev = (xcurrent-xmin)*xscale+XMIN;
		ydev = (ycurrent-ymin)*yscale+YMIN;
		putcoord(xdev,ydev);
                break;
        case 't':       /* put a label in the graph */
		gets(label);
		fprintf(stderr,"Can't do labels yet\n");
		/* move a little because plotter wants to be
		  at lower left of first character, UNIX puts point
		  at center */
		/* ETX = 3 is  default label terminator */
                break;
        case 'a':       /* draw an arc */
                fprintf(stderr,"don't know how to draw arcs\n");
                for(i=0;i<6;i++)(void)getcoord();       /* flush it */
                break;
        case 'c':       /* draw a circle */
                fprintf(stderr,"don't know how to draw circles\n");
                for(i=0;i<3;i++)(void)getcoord();       /* flush it */
                break;
        case 'f':       /* change linestyle */
		gets(label);
		if(strcmp(label,"solid")==0)
			{putchar(ESC);putchar('`');}
		else if(strcmp(label,"dotted")==0)
			{putchar(ESC);putchar('a');}
		else if(strcmp(label,"shortdashed")==0)
			{putchar(ESC);putchar('c');}
		else if(strcmp(label,"longdashed")==0)
			{putchar(ESC);putchar('d');}
		else if(strcmp(label,"dotdashed")==0)
			{putchar(ESC);putchar('b');}
		break;
        default: fprintf(stderr,"unknown code = %c\n",code);
    }

    /* leave the thing in ALPHA mode */
    putchar(ALPHA); mode=ALPHA;

}
 
double getcoord(){
int c,d;
   c = getchar();
    d = getchar();
    return( (double)((d<<8)|c) );
}

/* put two coordinates in Vi550 language */
putcoord(x,y) int x,y;{
    putchar((y>>5&037)|0040);	/* high y bits */
    putchar((y   &037)|0140);	/*  low y bits */
    putchar((x>>5&037)|0040);	/* high x bits */
    putchar((x   &037)|0100);	/*  low x bits */
}
