/**********************  xplot.c ******************************************
Filter on X_Window for axis.
One can switch the WhitePixel and BlackPixel.
The graphics window will disappear if one hits "q" while in this window.
 
written by Weiqiang Liu 5/4/1989
modified by Doug Toussaint 5/29/89
*/

/* To compile:  cc -O -o xplot xplot.c -lm -lX11  (Ultrix) */

/* On the SUN IPC: The include files needed by xplot
(X11/Xlib.h X11/Xutil.h X11/Xos.h) are in
/usr/openwin/share/include on the SUN IPC.   3/27/91 (Claude Bernard) */
 
#define F_GROUND BlackPixel
#define B_GROUND WhitePixel
 
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
 
#include <stdio.h>
 
/* On Ultrix, Decstation 3100, this icon is in
   /usr/include/mit/X11/bitmaps/icon, and can just be included */
/*#include <mit/X11/bitmaps/icon>*/
#define icon_width 16
#define icon_height 16
static char icon_bits[] = {
   0xff, 0xff, 0xab, 0xaa, 0x55, 0xd5, 0xab, 0xaa, 0x05, 0xd0, 0x0b, 0xa0,
   0x05, 0xd0, 0x0b, 0xa0, 0x05, 0xd0, 0x0b, 0xa0, 0x05, 0xd0, 0x0b, 0xa0,
   0x55, 0xd5, 0xab, 0xaa, 0x55, 0xd5, 0xff, 0xff};
 
#define BITMAPDEPTH 1
#define SMALL 1
#define OK 0
 
/* These are used as arguments to nearly every Xlib routine, so it saves
 * routine arguments to declare them global.  If there were
 * additional source files, they would be declared extern there. */
Display *display;
int screen;
char *display_name = NULL;
char display_name_space[128];	/* space for constructing display name */

char *index(),*getenv();
 
void main(argc, argv)
int argc;
char **argv;
{
    Window win;
    unsigned int width, height, x = 0, y = 0;     /* window size and position */
    unsigned int border_width = 4;    /* four pixels */
    unsigned int display_width, display_height;
    unsigned int icon_width_t, icon_height_t;
    char *window_name = "Axis_XWindow";
    char *icon_name = "";
    Pixmap icon_pixmap;
    XSizeHints size_hints;
    XIconSize *size_list;
    int count,i,j;
    XEvent report;
    KeySym ks;
    char buf[1];
    GC gc[5];
    XFontStruct *font_info;
    int window_size = 0;    /* OK, or too SMALL to display contents */
    XRectangle rectangle;    /* for exposure event processing */
    char *ps,*temp,*realloc(),*malloc();

    /* Set default display name.  This may be changed by get_opt. */
    /* if the environment variable DISPLAY is set, use it.  Else getenv()
       returns NULL, in which case we use :0.0.
       This lets Xlib pick the fastest transport. */
    /* alternatively, we might try unix:0.0 or hostname:0.0 */
    display_name=getenv("DISPLAY");
    if(display_name==NULL){
        /*gethostname(display_name_space,128);*/
        /* actually hostnames limited to 32 characters */
        strcpy(display_name_space,":0.0");
        display_name = display_name_space;
    }
 
    /* get command line options */
    get_opt(argc,argv);

    /* read-in the graphic content */
    ps = (char *)malloc(10000*sizeof(char));
    j=0;
    while( (i = getchar()) != EOF){
        ps[j++] = (unsigned char)i;
        if(j % 10000 == 0){
            temp = (char *)realloc(ps,(10000+j)*sizeof(char));
            ps = temp;
        }
    }
    ps[j++] = 'Q';
 
    /* connect to X server */
    if ( (display=XOpenDisplay(display_name)) == NULL )
    {
        (void) fprintf( stderr, "xplot: cannot connect to X server %s\n",
	    XDisplayName(display_name));
        exit( -1 );
    }
 
    /* get screen size from display structure macro */
    screen = DefaultScreen(display);
    display_width = DisplayWidth(display, screen);
    display_height = DisplayHeight(display, screen);
 
    /* note that x and y are 0, since the default position of the window
     * is the top left corner of the root. This is fine since the window
     * manager often allows the user to position the window before mapping it.
 */
 
    /* size window with enough room for text */
    /**width = 2*display_width/3, height = 2*display_height/3;**/
    width = display_width/2, height = display_height/2;
 
    /* create opaque window */
    win = XCreateSimpleWindow(display, RootWindow(display,screen),
        x, y, width,height, border_width,
        F_GROUND(display,screen), B_GROUND(display,screen));
 
    /* Get available icon sizes from Window manager
    if (XGetIconSizes(display, RootWindow(display,screen), &size_list, &count)
 == 0)
    {
        (void) fprintf( stderr, "Window manager didn't set icon sizes - using
 default.\n");
        icon_width_t = 16;
        icon_height_t = 16;
    }
    else
    {
        while (icon_width_t < size_list->min_width
        icon_width_t = size_list->max_width;
        icon_height_t = size_list->max_height;
    }
    all the icon size stuff commented out */
 
    /* Create pixmap of depth 1 (bitmap) for icon */
    icon_pixmap = XCreateBitmapFromData(display, win, icon_bits,
 icon_width, icon_height);
 
    /* Set resize hints */
    size_hints.flags = PPosition | PSize | PMinSize;
    size_hints.x = x;
    size_hints.y = y;
    size_hints.width = width;
    size_hints.height = height;
    size_hints.min_width = 300;
    size_hints.min_height = 200;
 
    /* set Properties for window manager (always before mapping) */
    XSetStandardProperties(display, win, window_name, icon_name,
        icon_pixmap, argv, argc, &size_hints);
 
    /* Select event types wanted */
    XSelectInput(display, win,
	ExposureMask | KeyPressMask | StructureNotifyMask);
 
    load_font(&font_info);
 
    /* create GC for text and drawing */
    getGC(win, gc, font_info);
 
    /* Display window */
    XMapWindow(display, win);
 
    /* printf("%d\n",XDoesBackingStore(XScreenOfDisplay(display,screen)));*/
 
    /* get events, use first to display text and graphics */
    while (1)  {
        XNextEvent(display, &report);
        switch  (report.type) {
        case Expose:
            /* if this is the last contiguous expose
             * in a group, draw the window */
            if (report.xexpose.count == 0) {
                /* if window too small to use */
                while (XCheckTypedEvent(display, Expose, &report));
                if (window_size == SMALL)
                    TooSmall(win, gc[0], font_info);
                else {
                        draw_graphics(win, gc, font_info, width, height,ps);
                }
            }
            break;
        case ConfigureNotify:
            /* window has been resized, change width and
             * height to send to place_text and place_graphics
             * in next Expose */
            width = report.xconfigure.width;
            height = report.xconfigure.height;
            if ((width < size_hints.min_width) ||
		(height < size_hints.min_height))
                window_size = SMALL;
            else
                window_size = OK;
            break;
        case ButtonPress:
            /* trickle down into KeyPress (no break) */
        case KeyPress:
	    /* Figure out what key was pressed, quit on 'q' or 'Q' */
	    count = XLookupString((XKeyEvent *)(&report),buf,100,&ks,NULL);
	    if( count==1 ){
		if(buf[0]=='q' || buf[0]=='Q'){
                    XUnloadFont(display, font_info->fid);
                    for(i=0;i<5;i++)XFreeGC(display, gc[i]);
                    XCloseDisplay(display);
                    exit(1);
		}
		else fprintf(stderr,"Use 'q' to quit xplot\n");
	    }
	break;
        default:
            /* all events selected by StructureNotifyMask
             * except ConfigureNotify are thrown away here,
             * since nothing is done with them */
            break;
        } /* end switch */
    } /* end while */
}
 
getGC(win, gc, font_info)
Window win;
GC gc[];
XFontStruct *font_info;
{
    unsigned long valuemask = 0; /* ignore XGCvalues and use defaults */
    XGCValues values;
    unsigned int line_width = 1;
    int line_style;
    int cap_style = CapRound;
    int join_style = JoinRound;
        int dash_offset = 0;
        static char dotted[] = {1, 3};
        static char shortdashed[] = {3, 4};
        static char longdashed[] = {9, 4};
        static char dotdashed[] = {4, 3, 1, 3};
 
    /* solid */
    line_style = LineSolid;
    gc[0] = XCreateGC(display, win, valuemask, &values);
    XSetFont(display, gc[0], font_info->fid);
    XSetForeground(display, gc[0], F_GROUND(display,screen));
    XSetLineAttributes(display, gc[0], line_width, line_style, cap_style,
 join_style);
 
    /* dotted */
    line_style = LineOnOffDash;
    gc[1] = XCreateGC(display, win, valuemask, &values);
    XSetFont(display, gc[1], font_info->fid);
    XSetForeground(display, gc[1], F_GROUND(display,screen));
    XSetLineAttributes(display, gc[1], line_width, line_style, cap_style,
 join_style);
    XSetDashes(display, gc[1], dash_offset, dotted, 2);
 
    /* shortdashed */
    line_style = LineOnOffDash;
    gc[2] = XCreateGC(display, win, valuemask, &values);
    XSetFont(display, gc[2], font_info->fid);
    XSetForeground(display, gc[2], F_GROUND(display,screen));
    XSetLineAttributes(display, gc[2], line_width, line_style, cap_style,
 join_style);
    XSetDashes(display, gc[2], dash_offset, shortdashed, 2);
 
    /* longdashed */
    line_style = LineOnOffDash;
    gc[3] = XCreateGC(display, win, valuemask, &values);
    XSetFont(display, gc[3], font_info->fid);
    XSetForeground(display, gc[3], F_GROUND(display,screen));
    XSetLineAttributes(display, gc[3], line_width, line_style, cap_style,
 join_style);
    XSetDashes(display, gc[3], dash_offset, longdashed, 2);
 
    /* dotdashed */
    line_style = LineOnOffDash;
    gc[4] = XCreateGC(display, win, valuemask, &values);
    XSetFont(display, gc[4], font_info->fid);
    XSetForeground(display, gc[4], F_GROUND(display,screen));
    XSetLineAttributes(display, gc[4], line_width, line_style, cap_style,
 join_style);
    XSetDashes(display, gc[4], dash_offset, dotdashed, 4);
}
 
load_font(font_info)
XFontStruct **font_info;
{
    char *fontname = "fixedwidthfont";
 
    /* Access font */
    if ((*font_info = XLoadQueryFont(display,fontname)) == NULL)
    {
        (void) fprintf( stderr, "xplot: Cannot open fixedwidthfont font\n");
        exit( -1 );
    }
}
 
TooSmall(win, tgc, font_info)
Window win;
GC tgc;
XFontStruct *font_info;
{
    char *string1 = "Too Small";
    int y_offset, x_offset;
 
    y_offset = font_info->max_bounds.ascent + 2;
    x_offset = 2;
 
    /* output text, centered on each line */
    XDrawString(display, win, tgc, x_offset, y_offset, string1, strlen
(string1));
}
draw_graphics(win, gc, font_info, window_width, window_height,pp)
Window win;
GC gc[];
XFontStruct *font_info;
unsigned int window_width, window_height;
unsigned char *pp;
{
    float getcoord();
    int i,code,slen,swidth,dots,xplot,yplot,xlast,ylast;
    float x,y,xmin,xmax,ymin,ymax,xdel,ydel,xcurrent,ycurrent,xscale,yscale;
    char label[128];
    unsigned int line_width = 1;
    int line_style;
    int cap_style = CapRound;
    int j,join_style = JoinRound;
    GC tgc = gc[0];
    char *sgets();
 
    dots = window_width > window_height ? window_height : window_width;
    xmin = 0.;   ymin = 0.;
    xmax = 1000.;   ymax = 1000.;
    xdel = (xmax-xmin)/dots;
    ydel = (ymax-ymin)/dots;
    xscale = 1./xdel;
    yscale = 1./ydel;
 
    j=0;
    /* now begin translating input file to Xwindow calls */
    /* 'Q' is fake EOF code */
    while( (code = pp[j++] ) != 'Q'){
        switch(code){
        case 'e':
        XClearWindow(display,win);    /* erase screen */
                break;
        case 's': xmin = getcoord(pp,&j);    /* set up plotting window */
                ymin = getcoord(pp,&j);
                xmax = getcoord(pp,&j);
                ymax = getcoord(pp,&j);
        xdel = (xmax-xmin)/dots;
        ydel = (ymax-ymin)/dots;
        xscale = 1./xdel;
        yscale = 1./ydel;
                break;
        case 'm':                       /* read in current point */
                xcurrent = getcoord(pp,&j);
                ycurrent = getcoord(pp,&j);
        xplot = (xcurrent-xmin)*xscale;
        yplot = (ymax-ycurrent)*yscale;
        /*XDrawPoint(display,win,tgc,xplot,yplot);*/
                break;
        case 'p':                       /* read in current point */
                xcurrent = getcoord(pp,&j);
                ycurrent = getcoord(pp,&j);
        xplot = (xcurrent-xmin)*xscale;
        yplot = (ymax-ycurrent)*yscale;
        XDrawPoint(display,win,tgc,xplot,yplot);
                break;
        case 'l':                       /* plot a line  */
                xcurrent = getcoord(pp,&j);
                ycurrent = getcoord(pp,&j);
        xlast = (xcurrent-xmin)*xscale;
        ylast = (ymax-ycurrent)*yscale;
                xcurrent = getcoord(pp,&j);
                ycurrent = getcoord(pp,&j);
        xplot = (xcurrent-xmin)*xscale;
        yplot = (ymax-ycurrent)*yscale;
        XDrawLine(display,win,tgc,xlast,ylast,xplot,yplot);
                break;
        case 'n':                       /* line from current to next*/
                xcurrent = getcoord(pp,&j);
                ycurrent = getcoord(pp,&j);
        xplot = (xcurrent-xmin)*xscale;
        yplot = (ymax-ycurrent)*yscale;
        XDrawLine(display,win,tgc,xlast,ylast,xplot,yplot);
                break;
        case 't':       /* put a label in the graph */
            sgets(label,pp,&j);
            slen = strlen(label);
            swidth = XTextWidth(font_info, label, slen);
            xplot = (xcurrent-xmin)*xscale;
            yplot = (ymax-ycurrent)*yscale;
            XDrawString(display,win,tgc,xplot,yplot,label,slen);
	    xcurrent += swidth/xscale;
                break;
        case 'a':       /* draw an arc */
                fprintf(stderr,"don't know how to draw arcs\n");
                for(i=0;i<6;i++)(void)getcoord(pp,&j);       /* flush it */
                break;
        case 'c':       /* draw a circle */
                fprintf(stderr,"don't know how to draw circles\n");
                for(i=0;i<3;i++)(void)getcoord(pp,&j);       /* flush it */
                break;
        case 'f':       /* change linestyle */
        sgets(label,pp,&j);
        if(strcmp(label,"solid")==0)tgc=gc[0];
        else if(strcmp(label,"dotted")==0)tgc=gc[1];
        else if(strcmp(label,"shortdashed")==0)tgc=gc[2];
        else if(strcmp(label,"longdashed")==0)tgc=gc[3];
        else if(strcmp(label,"dotdashed")==0)tgc=gc[4];
                break;
        default: fprintf(stderr,"unknown code = %c\n",code);
        }
    xlast = xplot;   ylast = yplot;
    }
 
}
 
float getcoord(pp,pj)
unsigned char *pp;
int *pj;
{
   int c,d;
 
   c = pp[(*pj)++];
   d = pp[(*pj)++];
    return( (float)((d<<8)|c) );
}
char *sgets(s,pp,pj)
char *s;
unsigned char *pp;
int *pj;
{
    register int c;
    register char *cs;
 
    cs = s;
    while((c = pp[(*pj)++] ) != EOF)
       if((*(cs++) = c) == '\n')
         break;
    *(--cs) = '\0';
         return( (c == EOF && cs == s)? NULL : s );
}

/* read command line options */
get_opt(argc,argv) int argc; char **argv; {
int i;
float linewidth,scale,x;
char *tp;
   for(i=1;i<argc;i++){
       if(strcmp(argv[i],"-w")==0){
	   /* THIS IS NOT YET IMPLEMENTED */
	   if(i==argc-1) fprintf(stderr,"Missing xplot linewidth option\n");
	   else if(sscanf(argv[++i],"%e",&x) == 1){
	       linewidth=x;
	   }
	   else fprintf(stderr,"Bad xplot linewidth option %s\n",argv[i]);
       }
       else if(strcmp(argv[i],"-d")==0){
	   /* display name may be full, ie "hostname:0.0", or just hostname,
	      in which case we append ":0.0" */
	   if(i==argc-1) fprintf(stderr,"Missing xplot display_name option\n");
	   else{
	        strncpy(display_name_space,argv[++i],128);
	        display_name_space[128-1-4]=NULL;
	        display_name = display_name_space;
		if(index(display_name_space,':')==NULL)
		    strcat(display_name_space,":0.0");
	   }
       }
       else fprintf(stderr,"Illegal xplot option %s\n",argv[i]);
   }
}
